"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const fse = require("fs-extra");
const utils = require("./utils");
const path_1 = require("path");
const glob = require("glob");
function readFile(filePath) {
    return new Promise((resolve, reject) => {
        fs.readFile(filePath, 'utf8', (err, data) => {
            if (err) {
                reject(err);
            }
            resolve(data);
        });
    });
}
exports.readFile = readFile;
function writeFile(filePath, data) {
    return new Promise((resolve, reject) => {
        fs.writeFile(filePath, data, 'utf8', (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
    });
}
exports.writeFile = writeFile;
function writeBufferToFile(filePath, data) {
    return new Promise((resolve, reject) => {
        fs.writeFile(filePath, data, (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
    });
}
exports.writeBufferToFile = writeBufferToFile;
function ensureDirectory(dirPath) {
    return new Promise((resolve, reject) => {
        fse.ensureDir(dirPath, (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
    });
}
exports.ensureDirectory = ensureDirectory;
function readDir(dirPath) {
    return new Promise((resolve, reject) => {
        fs.readdir(dirPath, (err, files) => {
            if (err) {
                reject(err);
            }
            resolve(files);
        });
    });
}
exports.readDir = readDir;
function globSearch(globString) {
    return new Promise((resolve, reject) => {
        glob(globString, (err, matches) => {
            if (err) {
                reject(err);
            }
            resolve(matches);
        });
    });
}
exports.globSearch = globSearch;
function writeJsonFile(filePath, data = {}) {
    return writeFile(filePath, JSON.stringify(data, null, 2));
}
exports.writeJsonFile = writeJsonFile;
function readJsonFile(filePath) {
    return readFile(filePath).then(data => JSON.parse(data));
}
exports.readJsonFile = readJsonFile;
function existsSync(filePath) {
    return fs.existsSync(filePath);
}
exports.existsSync = existsSync;
function exists(filePath) {
    return new Promise((resolve, reject) => {
        fs.exists(filePath, (exists) => resolve(exists));
    });
}
exports.exists = exists;
function removeFile(filePath) {
    return new Promise((resolve, reject) => {
        fs.unlink(filePath, (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
    });
}
exports.removeFile = removeFile;
function writeTarball(packageName, attachments) {
    let name = Object.keys(attachments)[0];
    let destFile = utils.getFilePath(`tarballs/${packageName}/${name}`);
    return ensureDirectory(path_1.dirname(destFile))
        .then(() => {
        let buf = Buffer.from(attachments[name].data, 'base64');
        return writeBufferToFile(destFile, buf);
    });
}
exports.writeTarball = writeTarball;
