"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const path_1 = require("path");
const fs_1 = require("./fs");
class Package {
    constructor(data, name) {
        if (data) {
            this.data = data;
            this.packageRoot = utils_1.getFilePath(`packages/${this.data.name}`);
            this.tarballRoot = utils_1.getFilePath(`tarballs/${this.data.name}`);
        }
        if (name) {
            this.packageRoot = utils_1.getFilePath(`packages/${name}`);
            this.tarballRoot = utils_1.getFilePath(`tarballs/${name}`);
        }
        this.pkgJsonPath = path_1.join(this.packageRoot, 'package.json');
    }
    init() {
        return this.ensureRootFolders()
            .then(() => this.initDataFromPkgJson());
    }
    getData() {
        if (!this.data) {
            return this.initDataFromPkgJson()
                .then(() => this.data);
        }
        else {
            return Promise.resolve(this.data);
        }
    }
    initDataFromPkgJson() {
        return fs_1.readJsonFile(this.pkgJsonPath)
            .then((jsonData) => this.data = jsonData)
            .catch(err => console.error(err));
    }
    initPkgJsonFromData() {
        let data = this.data;
        data._attachments = {};
        return this.ensureRootFolders()
            .then(() => fs_1.exists(this.pkgJsonPath))
            .then(oldVersionExists => {
            if (oldVersionExists) {
                return fs_1.readJsonFile(this.pkgJsonPath)
                    .then((jsonData) => {
                    if (jsonData) {
                        Object.keys(jsonData.versions).forEach(v => {
                            if (!data.versions[v]) {
                                data.versions[v] = jsonData.versions[v];
                            }
                        });
                    }
                    return fs_1.writeJsonFile(this.pkgJsonPath, data);
                });
            }
            else {
                return fs_1.writeJsonFile(this.pkgJsonPath, data);
            }
        });
    }
    saveTarballFromData() {
        return this.ensureRootFolders()
            .then(() => {
            let latestVersion = this.data['dist-tags'].latest;
            let tarballPath = this.tarballRoot + '/' + this.data.name + '-' + latestVersion + '.tgz';
            return fs_1.writeTarball(this.data.name, this.data._attachments);
        });
    }
    ensureRootFolders() {
        return fs_1.ensureDirectory(this.packageRoot)
            .then(() => fs_1.ensureDirectory(this.tarballRoot))
            .catch(err => console.error(err));
    }
}
exports.Package = Package;
