"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const bodyParser = require("body-parser");
const logger = require("./logger");
const routes = require("./routes");
const web = require("./web");
const auth = require("./auth");
const path_1 = require("path");
exports.router = express.Router();
function index(req, res) {
    return res.status(200).sendFile(path_1.resolve(__dirname, '../app/index.html'));
}
exports.router.use(bodyParser.json({ limit: '100Mb' }));
exports.router.use(auth.middleware);
exports.router.use(logger.middleware);
exports.router.use('/css', express.static(path_1.resolve(__dirname, '../app/css'), { index: false }));
exports.router.use('/js', express.static(path_1.resolve(__dirname, '../app/js'), { index: false }));
exports.router.use('/images', express.static(path_1.resolve(__dirname, '../app/images'), { index: false }));
exports.router.use('/css/fonts', express.static(path_1.resolve(__dirname, '../app/fonts'), { index: false }));
exports.router.get('/user/login', index);
exports.router.get('/package/:package(*)', index);
exports.router.get('/api/package/get-random', web.getRandomPackages);
exports.router.get('/api/package/:package(*)', web.getPackage);
exports.router.get('/api/package/search', web.searchPackages);
exports.router.post('/api/user/login', web.login);
exports.router.post('/api/user/organizations', web.getUserOrganizations);
exports.router.get('/user/login', index);
exports.router.post('/user/add', web.newUser);
exports.router.post('/org/add', web.newOrganization);
exports.router.post('/team/add', web.newTeam);
exports.router.post('/org/user/add', web.addUserToOrganization);
exports.router.post('/team/user/add', web.addUserToTeam);
exports.router.post('/team/delete', web.deleteTeam);
exports.router.post('/team/user/delete', web.deleteUserFromTeam);
exports.router.post('/org/user/delete', web.deleteUserFromOrganization);
exports.router.post('/org/delete', web.deleteOrganization);
exports.router.post('/org/user/role', web.changeUserRole);
exports.router.post('/pkg/publish', web.publishPackage);
exports.router.put(/\/\-\/user\/org\.couchdb\.user\:(.*)/, routes.doAuth);
exports.router.get(/\/\-\/user\/org\.couchdb\.user\:(.*)/, routes.getUser);
exports.router.delete('/-/user/token/:token', routes.logout);
exports.router.get('/-/whoami', routes.whoami);
exports.router.get('/-/ping', routes.ping);
exports.router.get('/:package/:version?', routes.getPackage);
exports.router.get('/:package(*)/-/:tarball(*)', routes.getTarball);
exports.router.get('/-/package/(*)/collaborators?', auth.hasAccess, routes.getCollaborators);
exports.router.get('/-/org/*/package*', auth.hasAccess, routes.organizationAccess);
exports.router.get('/-/team/*/package*', auth.hasAccess, routes.organizationAccess);
exports.router.get('/-/v1/search', auth.hasAccess, routes.search);
exports.router.get('/-/_view/starredByUser?', auth.hasAccess, routes.starredByUser);
exports.router.delete('/-/team/*/package*', auth.hasAccess, routes.setOrganizationAccess);
exports.router.put('/-/team/*/package*', auth.hasAccess, routes.setOrganizationAccess);
exports.router.put('/:package/:_rev/:revision?', auth.hasAccess, routes.updatePackageOwner);
exports.router.put('/:package/:revision?', auth.hasAccess, routes.updatePackage);
exports.router.post('/-/package/*/access', routes.setPackageAccess);
exports.router.all('/*', index);
