"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const cors = require("cors");
const router_1 = require("./router");
const logger = require("./logger");
const fs = require("./fs");
const utils = require("./utils");
const storage_1 = require("./storage");
function start(dir, test) {
    let app = express();
    utils.setHomeDir(dir);
    initMorose()
        .then(() => utils.getConfig())
        .then(config => {
        app.use(cors());
        app.use(router_1.router);
        app.listen(config.port, () => logger.info(`server running on port ${config.port}`));
    });
}
exports.start = start;
function initMorose() {
    let root = utils.getRootDir();
    return fs.exists(root).then(exists => {
        if (exists) {
            return storage_1.initializeStorage();
        }
        else {
            return fs.ensureDirectory(root)
                .then(() => utils.writeInitConfig())
                .then(() => fs.ensureDirectory(utils.getFilePath('packages')))
                .then(() => fs.ensureDirectory(utils.getFilePath('logs')))
                .then(() => fs.ensureDirectory(utils.getFilePath('tarballs')))
                .then(() => storage_1.initializeStorage())
                .then(() => logger.info(`morose successfully initialized.`))
                .catch(err => console.error(err));
        }
    });
}
