"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const path_1 = require("path");
const fs_1 = require("./fs");
const fs = require("fs");
const logger = require("./logger");
const auth_1 = require("./auth");
let home = os_1.homedir();
function setHomeDir(path) {
    home = path;
}
exports.setHomeDir = setHomeDir;
function getRootDir() {
    return home;
}
exports.getRootDir = getRootDir;
function getConfigPath() {
    return path_1.join(getRootDir(), 'config.json');
}
exports.getConfigPath = getConfigPath;
function getAuthPath() {
    return path_1.join(getRootDir(), 'auth.json');
}
exports.getAuthPath = getAuthPath;
function getConfig() {
    let configPath = getConfigPath();
    return JSON.parse(fs.readFileSync(configPath, 'utf8'));
}
exports.getConfig = getConfig;
function getAuth() {
    let path = getAuthPath();
    return JSON.parse(fs.readFileSync(path, 'utf8'));
}
exports.getAuth = getAuth;
function getFilePath(relativePath) {
    return path_1.join(getRootDir(), relativePath);
}
exports.getFilePath = getFilePath;
function writeInitConfig() {
    let secret = Math.random().toString(36).substr(2, 10);
    let data = {
        port: 10000,
        ssl: false,
        secret: secret,
        upstream: 'https://registry.npmjs.org',
        useUpstream: true,
        saveUpstreamPackages: true
    };
    return fs_1.writeJsonFile(getConfigPath(), data).then(() => writeAuth(secret));
}
exports.writeInitConfig = writeInitConfig;
function writeAuth(secret) {
    let password = 'admin';
    let data = {
        organizations: [
            {
                name: 'bleenco',
                teams: [{ name: 'developers', members: ['admin'] }],
                members: [{ name: 'admin', role: 'owner' }]
            }
        ],
        users: [
            {
                name: 'admin', password: auth_1.generateHash(password, secret), fullName: '', email: '', tokens: []
            }
        ],
        packages: []
    };
    logger.info(`initializing \`admin\` user with password \`${password}\``);
    return fs_1.writeJsonFile(getAuthPath(), data);
}
exports.writeAuth = writeAuth;
function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
exports.getRandomInt = getRandomInt;
function getHumanSize(bytes, decimals = 2) {
    if (!bytes) {
        return '0 Bytes';
    }
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
    const k = 1000;
    const i = Math.floor(Math.log(bytes) / Math.log(k));
    return `${parseFloat((bytes / Math.pow(k, i)).toFixed(decimals))} ${sizes[i]}`;
}
exports.getHumanSize = getHumanSize;
