"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require("jsonwebtoken");
const storage_1 = require("./storage");
const utils_1 = require("./utils");
const auth = require("./auth");
const fs_1 = require("./fs");
function getRandomPackages(req, res) {
    let max = storage_1.storage.length;
    let pkgs = [...Array(max < 9 ? max : 9).keys()]
        .map(() => storage_1.storage[utils_1.getRandomInt(0, max - 1)])
        .filter(Boolean);
    return res.status(200).json(pkgs);
}
exports.getRandomPackages = getRandomPackages;
function getPackage(req, res) {
    let pkgName = req.params.package;
    let index = storage_1.storage.findIndex(pkg => pkg.name === pkgName);
    if (index !== -1) {
        return res.status(200).json({ status: true, data: storage_1.storage[index] });
    }
    else {
        return res.status(200).json({ status: false });
    }
}
exports.getPackage = getPackage;
function searchPackages(req, res) {
    let pkgs = storage_1.storage
        .map(pkg => pkg && pkg.name.indexOf(req.query.keyword) !== -1 ? pkg : null)
        .filter(Boolean);
    return res.status(200).json(pkgs);
}
exports.searchPackages = searchPackages;
function login(req, res) {
    let config = utils_1.getConfig();
    let { username, password } = req.body;
    if (auth.checkUser(username, password)) {
        let token = jwt.sign({ name: username }, config.secret);
        return res.status(200).json({ auth: true, token: token });
    }
    else {
        return res.status(200).json({ auth: false });
    }
}
exports.login = login;
function getUserOrganizations(req, res) {
    let username = req.body.username;
    let auth = utils_1.getAuth();
    let orgs = auth.organizations
        .filter(org => org.members.findIndex(u => u.name === username) !== -1);
    return res.status(200).json(orgs);
}
exports.getUserOrganizations = getUserOrganizations;
function newUser(req, res) {
    let authObj = utils_1.getAuth();
    let config = utils_1.getConfig();
    auth.newUser(req.body, authObj, config)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.newUser = newUser;
function newOrganization(req, res) {
    let authObj = utils_1.getAuth();
    auth.newOrganization(req.body.organization, req.body.username, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.newOrganization = newOrganization;
function newTeam(req, res) {
    let authObj = utils_1.getAuth();
    auth.newTeam(req.body, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.newTeam = newTeam;
function addUserToOrganization(req, res) {
    let authObj = utils_1.getAuth();
    auth.addUserToOrganization(req.body.username, req.body.organization, req.body.role, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.addUserToOrganization = addUserToOrganization;
function addUserToTeam(req, res) {
    let authObj = utils_1.getAuth();
    auth.addUserToTeam(req.body, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.addUserToTeam = addUserToTeam;
function deleteTeam(req, res) {
    let authObj = utils_1.getAuth();
    auth.deleteTeam(req.body.team, req.body.organization, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.deleteTeam = deleteTeam;
function deleteUserFromTeam(req, res) {
    let authObj = utils_1.getAuth();
    auth.deleteUserFromTeam(req.body, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.deleteUserFromTeam = deleteUserFromTeam;
function deleteUserFromOrganization(req, res) {
    let authObj = utils_1.getAuth();
    auth.deleteUserFromOrganization(req.body.username, req.body.organization, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.deleteUserFromOrganization = deleteUserFromOrganization;
function deleteOrganization(req, res) {
    let authObj = utils_1.getAuth();
    auth.deleteOrganization(req.body.organization, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.deleteOrganization = deleteOrganization;
function changeUserRole(req, res) {
    let authObj = utils_1.getAuth();
    auth.changeUserRole(req.body.username, req.body.organization, req.body.team, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.changeUserRole = changeUserRole;
function publishPackage(req, res) {
    let authObj = utils_1.getAuth();
    auth.publishPackage(req.body.pkgName, req.body.username, req.body.organization, req.body.teams, req.body.version, authObj)
        .then(auth => fs_1.writeJsonFile(utils_1.getAuthPath(), auth)
        .then(() => res.status(200).json({ data: true })))
        .catch(err => res.status(200).json({ message: err }));
}
exports.publishPackage = publishPackage;
